// Database.hpp
//
// 20000818 / dhartmei - initial revision
// 20000905 / dhartmei - added db based database
// 20001001 / dhartmei - db disabled for Linux compatibility
//

#ifndef DATABASE_HPP
#define DATABASE_HPP

#if (!__WIN32__ && !__Linux)
  #include <sys/types.h>
  #include <fcntl.h>
  #include <limits.h>
  #include <db.h>
  #include <unistd.h>
#endif
#include <string>
#include <map>
#include <vector>

// ----------------------------------------------------------------------------

class Database
  {
    public:
      Database() {}
      virtual ~Database() {}
      virtual void clear() {}
      virtual void load() {}
      virtual void save() const {}
      virtual std::string find(std::string key) const = 0;
      virtual void apropos(std::vector<std::string> words, std::vector<std::string> &keys) const = 0;
      virtual void insert(std::string key, std::string value) = 0;
      virtual void remove(std::string key) = 0;
      virtual unsigned size() const = 0;
      virtual void importFile(std::string filename) = 0;
      virtual void exportFile(std::string filename) const = 0;
    protected:
    private:
  };

// ----------------------------------------------------------------------------

class MemoryDatabase : public Database
  {
    public:
      MemoryDatabase(std::string filename);
      virtual ~MemoryDatabase();
      virtual void clear();
      virtual void load();
      virtual void save() const;
      virtual std::string find(std::string key) const;
      virtual void apropos(std::vector<std::string> words, std::vector<std::string> &keys) const;
      virtual void insert(std::string key, std::string value);
      virtual void remove(std::string key);
      virtual unsigned size() const { return entries.size(); }
      virtual void importFile(std::string filename);
      virtual void exportFile(std::string filename) const;
    protected:
    private:
      std::string filename;
      typedef std::map< std::string, std::string, std::less< std::string > > Map;
      Map entries;
  };

// ----------------------------------------------------------------------------

#if (!__WIN32__ && !__Linux__)

class FileDatabase : public Database
  {
    public:
      FileDatabase(std::string filename);
      virtual ~FileDatabase();
      virtual void clear();
      virtual void save() const;
      virtual std::string find(std::string key) const;
      virtual void apropos(std::vector<std::string> words, std::vector<std::string> &keys) const;
      virtual void insert(std::string key, std::string value);
      virtual void remove(std::string key);
      virtual unsigned size() const { return 0; }
      virtual void importFile(std::string filename);
      virtual void exportFile(std::string filename) const;
    protected:
    private:
      void open();
      void close();
      std::string filename;
      BTREEINFO btreeinfo;
      DB *db;
      mutable DBT dbtKey, dbtData;
  };

#endif

// ----------------------------------------------------------------------------

#endif
